#include <avr/io.h>
#include <avr/interrupt.h>
#include "Test.c"

#define CountTest 1000     //    .
#define MYUBRR (F_CPU/16/9600-1)

volatile uint32_t Tim_Counter=0;

ISR (TIMER1_OVF_vect) //   
{ 
   Tim_Counter++;
}


void Init(void)
{

  TCCR1A = 0x00;
  TIMSK1 =(1<<TOIE1); //      1.
  TCNT1 = 0x00;


  UBRR0H = (unsigned char)(MYUBRR>>8);
  UBRR0L = (unsigned char)MYUBRR;
  /*Enable transmitter */
  UCSR0B = (1<<TXEN0);
  /* Set frame format: 8data, 2stop*/
  UCSR0C = (1<<USBS0)|(3<<UCSZ00);
}


void SendStr(char *str)
{
    if (str)
    {
        while (*str)
        {
            while(!(UCSR0A & (1<<UDRE0)));
            UDR0 = *str++;
        }

    }
}

void SendNum(uint32_t num)
{
    char Buff[12];

    Buff[11]=0;
    uint8_t i;

    for (i=10; i>0; i--)
    {
        Buff[i] = (char)((num % 10)+0x30);
        num /= 10;
        if (num==0) break;
    }

    SendStr(&Buff[i]);
}

void SendRes(char *Str, uint32_t num)
{
    SendStr(Str);
    SendNum(num);
    while(!(UCSR0A & (1<<UDRE0)));
    UDR0 = 10;
}

int main(void)
{
    static uint16_t i;
    static int8_t x8=0;
    static int16_t x16=0;
    static int32_t x32=0;
    static float xf=0;
    static double xd=0;
	static uint32_t Result;

    Init();


    sei(); //  .


     // 8 .
    Tim_Counter=0;
    TCNT1 = 0;
    TCCR1B |= (1<<CS10); //  = 1.

    for(i=0; i<CountTest; i++)
    {
        x8 = Test8(TCNT1 % (uint16_t)x8);
    }

    TCCR1B &= ~((1<<CS12) | (1<<CS11) | (1<<CS10));  // .
    Result = (uint32_t)TCNT1 + ((uint32_t)0xFFFF * Tim_Counter);
    SendRes("1 byte = ", Result);



    // 16 .
    Tim_Counter=0;
    TCNT1 = 0;
    TCCR1B |= (1<<CS10); //  = 1.
    
    for(i=0; i<CountTest; i++)
    {
	    x16 = Test16((int16_t)TCNT1 % x16);
    }

    TCCR1B &= ~((1<<CS12) | (1<<CS11) | (1<<CS10));  // .
    Result = (uint32_t)TCNT1 + ((uint32_t)0xFFFF * Tim_Counter);
    SendRes("2 byte = ", Result);


    // 32 .
    Tim_Counter=0;
    TCNT1 = 0;
    TCCR1B |= (1<<CS10); //  = 1.
    
    for(i=0; i<CountTest; i++)
    {
	    x32 = Test32((int32_t)TCNT1 % x32);
    }

    TCCR1B &= ~((1<<CS12) | (1<<CS11) | (1<<CS10));  // .
    Result = (uint32_t)TCNT1 + ((uint32_t)0xFFFF * Tim_Counter);
    SendRes("4 byte = ", Result);


    // Float.
    Tim_Counter=0;
    TCNT1 = 0;
    TCCR1B |= (1<<CS10); //  = 1.
    
    for(i=0; i<CountTest; i++)
    {
	    xf = Test_f((float)TCNT1 / xf);
    }

    TCCR1B &= ~((1<<CS12) | (1<<CS11) | (1<<CS10));  // .
    Result = (uint32_t)TCNT1 + ((uint32_t)0xFFFF * Tim_Counter);
    SendRes("Float = ", Result);


    // Double.
    Tim_Counter=0;
    TCNT1 = 0;
    TCCR1B |= (1<<CS10); //  = 1.
    
    for(i=0; i<CountTest; i++)
    {
	    xd = Test_d((double)TCNT1 / xd);
    }

    TCCR1B &= ~((1<<CS12) | (1<<CS11) | (1<<CS10));  // .
    Result = (uint32_t)TCNT1 + ((uint32_t)0xFFFF * Tim_Counter);
    SendRes("Double = ", Result);


    while(1)
    {
    }

}
