/*
**
**                           Main.c
**
**
**********************************************************************/
/*
   Last committed:     $Revision: 00 $
   Last changed by:    $Author: $
   Last changed date:  $Date:  $
   ID:                 $Id:  $

**********************************************************************/
#include "stm32f0xx_conf.h"
#include "Test.c"


#define CountTest 1000     //    .

volatile uint32_t Tim_Counter=0;
uint32_t Results[10];

void TIM3_IRQHandler(void) //    TIM3.
{
    if (TIM_GetITStatus(TIM3, TIM_IT_Update) != RESET)
    {
        TIM_ClearITPendingBit(TIM3, TIM_IT_Update); //      .
        Tim_Counter++;
    }

}

void Init(void)
{
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    //  .
    TIM_TimeBaseInitTypeDef Tim;
    TIM_TimeBaseStructInit(&Tim);
    TIM_TimeBaseInit(TIM3, &Tim);

    TIM_ITConfig(TIM3, TIM_IT_Update, ENABLE); //     .
    NVIC_EnableIRQ(TIM3_IRQn);


    //  USART.

    RCC_USARTCLKConfig(RCC_USART1CLK_SYSCLK);  // USART    .

    GPIO_InitTypeDef GPIO;

    GPIO.GPIO_Pin   = GPIO_Pin_2;       //   PA2 (TXD).
    GPIO.GPIO_Mode  = GPIO_Mode_AF;     //       .
    GPIO.GPIO_OType = GPIO_OType_PP;    //   (. .   ).
    GPIO.GPIO_PuPd  = GPIO_PuPd_NOPULL; //   .
    GPIO.GPIO_Speed = GPIO_Speed_50MHz;

    GPIO_Init(GPIOA, &GPIO);            //  .

    GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_1); // GPIOA.2   USART1.

    USART_InitTypeDef USART;

    USART.USART_BaudRate = 9600;
    USART.USART_WordLength = USART_WordLength_8b;
    USART.USART_StopBits = USART_StopBits_1;
    USART.USART_Parity = USART_Parity_No;
    USART.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART.USART_Mode = USART_Mode_Tx;

    USART_Init(USART1, &USART);

    USART_Cmd(USART1, ENABLE);

}


void SendStr(char *str)
{
    if (str)
    {
        while (*str)
        {
            while (USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);
            USART_SendData(USART1, *str++);
        }

    }
}

void SendNum(uint32_t num)
{
    char Buff[12];

    Buff[11]=0;
    uint8_t i;

    for (i=10; i>0; i--)
    {
        Buff[i] = (char)((num % 10)+0x30);
        num /= 10;
        if (num==0) break;
    }

    SendStr(&Buff[i]);
}

void SendRes(char *Str, uint32_t num)
{
    SendStr(Str);
    SendNum(num);
    while (USART_GetFlagStatus(USART1, USART_FLAG_TC) == RESET);
    USART_SendData(USART1, 10);
}

int main(void)
{
    static uint16_t i;
    static int8_t x8=0;
    static int16_t x16=0;
    static int32_t x32=0;
    static float xf=0;
    static double xd=0;

    Init();

    // 8 .
    Tim_Counter=0;
    TIM3->CNT = 0;
    TIM_Cmd(TIM3, ENABLE);

    for(i=0; i<CountTest; i++)
    {
        x8 = Test8((int8_t)(TIM3->CNT % (uint16_t)x8));
    }

    TIM_Cmd(TIM3, DISABLE);
    Results[0] = ((uint32_t)TIM3->CNT) + (0xFFFF * Tim_Counter);
    SendRes("1 byte = ", Results[0]);


    // 16 .
    Tim_Counter=0;
    TIM3->CNT = 0;
    TIM_Cmd(TIM3, ENABLE);

    for(i=0; i<CountTest; i++)
    {
        x16 = Test16((int16_t)(TIM3->CNT % x16));
    }

    TIM_Cmd(TIM3, DISABLE);
    Results[1] = ((uint32_t)TIM3->CNT) + (0xFFFF * Tim_Counter);
    SendRes("2 byte = ", Results[1]);

    // 32 .
    Tim_Counter=0;
    TIM3->CNT = 0;
    TIM_Cmd(TIM3, ENABLE);

    for(i=0; i<CountTest; i++)
    {
        x32 = Test32(((int32_t)TIM3->CNT) % x32);
    }

    TIM_Cmd(TIM3, DISABLE);
    Results[2] = ((uint32_t)TIM3->CNT) + (0xFFFF * Tim_Counter);
    SendRes("4 byte = ", Results[2]);

    // float.
    Tim_Counter=0;
    TIM3->CNT = 0;
    TIM_Cmd(TIM3, ENABLE);

    for(i=0; i<CountTest; i++)
    {
        xf = Test_f((float)TIM3->CNT / xf);
    }

    TIM_Cmd(TIM3, DISABLE);
    Results[3] = ((uint32_t)TIM3->CNT) + (0xFFFF * Tim_Counter);
    SendRes("float = ", Results[3]);

    // double.
    Tim_Counter=0;
    TIM3->CNT = 0;
    TIM_Cmd(TIM3, ENABLE);

    for(i=0; i<CountTest; i++)
    {
        xd = Test_d((double)TIM3->CNT / xd);
    }

    TIM_Cmd(TIM3, DISABLE);
    Results[4] = ((uint32_t)TIM3->CNT) + (0xFFFF * Tim_Counter);
    SendRes("double = ", Results[4]);

    while(1)
    {
       __NOP;
    }
}

